//                                                                  
// Copyright  2013 Lidor Systems.
// All Rights Reserved.
//
// This SOFTWARE is provided "AS IS", WITHOUT WARRANTY OF ANY KIND.
// either express or implied.                        
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using LidorSystems.IntegralUI.Lists;
using LidorSystems.IntegralUI.Serialization;

namespace IntegralUI_WinForms_Samples
{
    public partial class Form1 : Form
    {
        #region Variables
        private int[] randomInts = null;
        private int nodeCount = 0;
        private String directoryName = "../../../Demo/";
        #endregion

        #region Initialization
        public Form1()
        {
            InitializeComponent();

            this.propertyGrid1.SelectedObject = this.treeListView1.ColorStyle;
            this.treeListView1.AllowSubItemSelection = true;
            this.treeListView1.SelectionMode = SelectionMode.MultiExtended;

            CreateLabelContent();

            InitList();
        }

        private void CreateLabelContent()
        {
            String content = "<div style=\"font:Verdana,9,r\"><p>This sample shows save custom color schemes to a XML file for later use.</p><br></br>";
            content += "<p>As demonstration three color schemes are created: red, green and blue, form which you can choose from.</p><br></br>";
            content += "<p>By changing color and format styles for different parts of the TreeListView you can also create your own color scheme and save it in custom XML file. Press CTRL+CLICK to select multiple subitems.</p><br></br>";
            content += "<p><table width=\"100%\" cellpadding=\"5\"><tr><td width=\"100%\" style=\"align:middleright;textcolor:DarkRed;hovertextcolor:Red\"><a href=\"www.lidorsystems.com/support/articles/winforms/treelistview/\">More Info ...</a></td></tr></table></p>";
            content += "</div>";

            this.labelInfo.Content = content;
        }

        private void labelInfo_ContentObjectClicked(object sender, LidorSystems.IntegralUI.ObjectClickEventArgs e)
        {
            if (e.Object is LinkLabel)
                System.Diagnostics.Process.Start((e.Object as LinkLabel).Links[0].LinkData.ToString());
        }

        private void InitList()
        {
            this.treeListView1.SuspendUpdate();

            nodeCount = 0;
            randomInts = CreateRandomInts();

            TreeListViewColumn column = null;
            for (int j = 1; j <= 3; j++)
            {
                column = new TreeListViewColumn("Header " + j.ToString(), "Footer " + j.ToString());
                column.Width = 120;

                this.treeListView1.Columns.Add(column);
            }


            CreateNode(this.treeListView1.RootNode, "", 0);

            this.treeListView1.ResumeUpdate();
        }

        private void CreateNode(TreeListViewNode parentNode, String prefix, int level)
        {
            if (level == 5)
                return;

            int limit = randomInts[nodeCount % 99];
            if (level == 0)
                limit = 100;

            TreeListViewNode node = null;
            TreeListViewSubItem subItem = null;

            for (int i = 1; i <= limit; i++)
            {
                node = new TreeListViewNode("Node " + prefix + i.ToString());

                for (int j = 0; j < this.treeListView1.Columns.Count; j++)
                {
                    subItem = new TreeListViewSubItem("Item " + i.ToString() + Convert.ToString(j + 1));
                    node.SubItems.Add(subItem);
                }

                parentNode.Nodes.Add(node);
                nodeCount++;

                CreateNode(node, prefix + i.ToString(), level + 1);
            }
        }

        private int[] CreateRandomInts()
        {
            int[] rand = new int[100];
            Random gen = new Random();
            for (int i = 0; i < rand.Length; i++)
                rand[i] = gen.Next(0, 5);

            return rand;
        }
        #endregion

        #region Load Color Scheme
        private void btnLoad_Click(object sender, System.EventArgs e)
        {
            // Suspend the control layout from updating to increase performance
            this.treeListView1.SuspendUpdate();

            TreeListViewSerializer serializer = new TreeListViewSerializer();

            if (rbRed.Checked)
                serializer.LoadFromXml(this.treeListView1, directoryName + "red.xml");
            else if (rbGreen.Checked)
                serializer.LoadFromXml(this.treeListView1, directoryName + "green.xml");
            else if (rbBlue.Checked)
                serializer.LoadFromXml(this.treeListView1, directoryName + "blue.xml");
            else
            {
                OpenFileDialog openDlg = new OpenFileDialog();
                openDlg.Filter = "XML files (*.xml)|*.xml";
                openDlg.InitialDirectory = Application.ExecutablePath;

                if (openDlg.ShowDialog() == DialogResult.OK)
                    serializer.LoadFromXml(this.treeListView1, openDlg.FileName);
            }

            // Resume the layout calculations and update the control
            this.treeListView1.ResumeUpdate();

            UpdatePropertyGrid();
        }
        #endregion

        #region Change Control Appearance
        private void btnSave_Click(object sender, System.EventArgs e)
        {
            SaveFileDialog saveDlg = new SaveFileDialog();
            saveDlg.Filter = "XML files (*.xml)|*.xml"; //|All files (*.*)|*.*";
            saveDlg.AddExtension = true;
            saveDlg.DefaultExt = ".xml";
            saveDlg.InitialDirectory = Application.ExecutablePath;

            if (saveDlg.ShowDialog() == DialogResult.OK)
            {
                TreeListViewSerializer serializer = new TreeListViewSerializer();
                serializer.SaveToXml(this.treeListView1, saveDlg.FileName, SerializationType.Style);
            }
        }

        private void cbPart_SelectedIndexChanged(object sender, EventArgs e)
        {
            cbState.Items.Clear();

            switch (cbPart.SelectedIndex)
            {
                case 1: // Column
                    cbState.Items.Add("Hovered");
                    cbState.Items.Add("Normal");
                    cbState.Items.Add("Selected");
                    cbState.SelectedIndex = 1;
                    cbState.Enabled = true;
                    break;

                case 2: // Item
                case 3: // SubItem
                    cbState.Items.Add("Disabled");
                    cbState.Items.Add("Focused");
                    cbState.Items.Add("Hovered");
                    cbState.Items.Add("Normal");
                    cbState.Items.Add("Selected");
                    cbState.SelectedIndex = 3;
                    cbState.Enabled = true;
                    break;

                default: // General
                    cbState.Text = String.Empty;
                    cbState.Enabled = false;
                    break;
            }

            UpdatePropertyGrid();
        }

        private void rbStyle_CheckedChanged(object sender, EventArgs e)
        {
            UpdatePropertyGrid();
        }

        private void cbState_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdatePropertyGrid();
        }

        private void UpdatePropertyGrid()
        {
            switch (cbPart.SelectedIndex)
            {
                case 1: // Column
                    if (rbColor.Checked)
                    {
                        // Color Styles
                        switch (cbState.SelectedIndex)
                        {
                            case 0: // Hovered State
                                this.propertyGrid1.SelectedObject = this.treeListView1.HoverColumnStyle;
                                break;

                            case 2: // Selected State
                                this.propertyGrid1.SelectedObject = this.treeListView1.SelectedColumnStyle;
                                break;

                            default: // Normal State
                                this.propertyGrid1.SelectedObject = this.treeListView1.NormalColumnStyle;
                                break;
                        }
                    }
                    else // Format Style
                        this.propertyGrid1.SelectedObject = this.treeListView1.ColumnFormatStyle;
                    break;

                case 2: // Item
                    if (rbColor.Checked)
                    {
                        // Color Styles
                        switch (cbState.SelectedIndex)
                        {
                            case 0: // Disabled State
                                this.propertyGrid1.SelectedObject = this.treeListView1.DisabledNodeStyle;
                                break;

                            case 1: // Focused State
                                this.propertyGrid1.SelectedObject = this.treeListView1.FocusedNodeStyle;
                                break;

                            case 2: // Hovered State
                                this.propertyGrid1.SelectedObject = this.treeListView1.HoverNodeStyle;
                                break;

                            case 4: // Selected State
                                this.propertyGrid1.SelectedObject = this.treeListView1.SelectedNodeStyle;
                                break;

                            default: // Normal State
                                this.propertyGrid1.SelectedObject = this.treeListView1.NormalNodeStyle;
                                break;
                        }
                    }
                    else // Format Style
                        this.propertyGrid1.SelectedObject = this.treeListView1.NodeFormatStyle;
                    break;

                case 3: // SubItem
                    if (rbColor.Checked)
                    {
                        // Color Styles
                        switch (cbState.SelectedIndex)
                        {
                            case 0: // Disabled State
                                this.propertyGrid1.SelectedObject = this.treeListView1.DisabledSubItemStyle;
                                break;

                            case 1: // Focused State
                                this.propertyGrid1.SelectedObject = this.treeListView1.FocusedSubItemStyle;
                                break;

                            case 2: // Hovered State
                                this.propertyGrid1.SelectedObject = this.treeListView1.HoverSubItemStyle;
                                break;

                            case 4: // Selected State
                                this.propertyGrid1.SelectedObject = this.treeListView1.SelectedSubItemStyle;
                                break;

                            default: // Normal State
                                this.propertyGrid1.SelectedObject = this.treeListView1.NormalSubItemStyle;
                                break;
                        }
                    }
                    else // Format Style
                        this.propertyGrid1.SelectedObject = this.treeListView1.SubItemFormatStyle;
                    break;

                default: // General
                    if (rbColor.Checked)
                        this.propertyGrid1.SelectedObject = this.treeListView1.ColorStyle;
                    else
                        this.propertyGrid1.SelectedObject = this.treeListView1.FormatStyle;
                    break;
            }
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            this.treeListView1.SuspendUpdate();

            this.treeListView1.ColorStyle.Reset();
            this.treeListView1.FormatStyle.Reset();
            this.treeListView1.ColumnFormatStyle.Reset();
            this.treeListView1.HoverColumnStyle.Reset();
            this.treeListView1.NormalColumnStyle.Reset();
            this.treeListView1.SelectedColumnStyle.Reset();
            this.treeListView1.DisabledNodeStyle.Reset();
            this.treeListView1.FocusedNodeStyle.Reset();
            this.treeListView1.HoverNodeStyle.Reset();
            this.treeListView1.NormalNodeStyle.Reset();
            this.treeListView1.SelectedNodeStyle.Reset();
            this.treeListView1.DisabledSubItemStyle.Reset();
            this.treeListView1.FocusedSubItemStyle.Reset();
            this.treeListView1.HoverSubItemStyle.Reset();
            this.treeListView1.NormalSubItemStyle.Reset();
            this.treeListView1.SelectedSubItemStyle.Reset();

            this.treeListView1.ResumeUpdate();

            UpdatePropertyGrid();
        }
        #endregion
    }
}