'                                                                  
' Copyright  2013 Lidor Systems.
' All Rights Reserved.
'
' This SOFTWARE is provided "AS IS", WITHOUT WARRANTY OF ANY KIND.
' either express or implied.                        
'

Imports LidorSystems.IntegralUI.Lists
Imports LidorSystems.IntegralUI.Serialization

Public Class Form1

#Region "Variables"
    Private randomInts As Integer() = Nothing
    Private nodeCount As Integer = 0
    Private directoryName As [String] = "../../../Demo/"
#End Region

#Region "Initialization"
    Public Sub New()
        InitializeComponent()

        Me.propertyGrid1.SelectedObject = Me.treeListView1.ColorStyle
        Me.treeListView1.AllowSubItemSelection = True
        Me.treeListView1.SelectionMode = SelectionMode.MultiExtended

        CreateLabelContent()

        InitList()
    End Sub

    Private Sub CreateLabelContent()
        Dim content As [String] = "<div style=""font:Verdana,9,r""><p>This sample shows save custom color schemes to a XML file for later use.</p><br></br>"
        content += "<p>As demonstration three color schemes are created: red, green and blue, form which you can choose from.</p><br></br>"
        content += "<p>By changing color and format styles for different parts of the TreeListView you can also create your own color scheme and save it in custom XML file. Press CTRL+CLICK to select multiple subitems.</p><br></br>"
        content += "<p><table width=""100%"" cellpadding=""5""><tr><td width=""100%"" style=""align:middleright;textcolor:DarkRed;hovertextcolor:Red""><a href=""www.lidorsystems.com/support/articles/winforms/treelistview/"">More Info ...</a></td></tr></table></p>"
        content += "</div>"

        Me.labelInfo.Content = content
    End Sub

    Private Sub labelInfo_ContentObjectClicked(ByVal sender As Object, ByVal e As LidorSystems.IntegralUI.ObjectClickEventArgs) Handles labelInfo.ContentObjectClicked
        If TypeOf e.[Object] Is LinkLabel Then
            System.Diagnostics.Process.Start(TryCast(e.[Object], LinkLabel).Links(0).LinkData.ToString())
        End If
    End Sub

    Private Sub InitList()
        Me.treeListView1.SuspendUpdate()

        nodeCount = 0
        randomInts = CreateRandomInts()

        Dim column As TreeListViewColumn = Nothing
        For j As Integer = 1 To 3
            column = New TreeListViewColumn("Header " & j.ToString(), "Footer " & j.ToString())
            column.Width = 120

            Me.treeListView1.Columns.Add(column)
        Next


        CreateNode(Me.treeListView1.RootNode, "", 0)

        Me.treeListView1.ResumeUpdate()
    End Sub

    Private Sub CreateNode(ByVal parentNode As TreeListViewNode, ByVal prefix As [String], ByVal level As Integer)
        If level = 5 Then
            Return
        End If

        Dim limit As Integer = randomInts(nodeCount Mod 99)
        If level = 0 Then
            limit = 100
        End If

        Dim node As TreeListViewNode = Nothing
        Dim subItem As TreeListViewSubItem = Nothing

        For i As Integer = 1 To limit
            node = New TreeListViewNode("Node " & Convert.ToString(prefix) & i.ToString())

            For j As Integer = 0 To Me.treeListView1.Columns.Count - 1
                subItem = New TreeListViewSubItem("Item " & i.ToString() & Convert.ToString(j + 1))
                node.SubItems.Add(subItem)
            Next

            parentNode.Nodes.Add(node)
            nodeCount += 1

            CreateNode(node, Convert.ToString(prefix) & i.ToString(), level + 1)
        Next
    End Sub

    Private Function CreateRandomInts() As Integer()
        Dim rand As Integer() = New Integer(99) {}
        Dim gen As New Random()
        For i As Integer = 0 To rand.Length - 1
            rand(i) = gen.[Next](0, 5)
        Next

        Return rand
    End Function
#End Region

#Region "Load Color Scheme"
    Private Sub btnLoad_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnLoad.Click
        ' Suspend the control layout from updating to increase performance
        Me.treeListView1.SuspendUpdate()

        Dim serializer As New TreeListViewSerializer()

        If rbRed.Checked Then
            serializer.LoadFromXml(Me.treeListView1, Convert.ToString(directoryName) & "red.xml")
        ElseIf rbGreen.Checked Then
            serializer.LoadFromXml(Me.treeListView1, Convert.ToString(directoryName) & "green.xml")
        ElseIf rbBlue.Checked Then
            serializer.LoadFromXml(Me.treeListView1, Convert.ToString(directoryName) & "blue.xml")
        Else
            Dim openDlg As New OpenFileDialog()
            openDlg.Filter = "XML files (*.xml)|*.xml"
            openDlg.InitialDirectory = Application.ExecutablePath

            If openDlg.ShowDialog() = DialogResult.OK Then
                serializer.LoadFromXml(Me.treeListView1, openDlg.FileName)
            End If
        End If

        ' Resume the layout calculations and update the control
        Me.treeListView1.ResumeUpdate()

        UpdatePropertyGrid()
    End Sub
#End Region

#Region "Change Control Appearance"
    Private Sub btnSave_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnSave.Click
        Dim saveDlg As New SaveFileDialog()
        saveDlg.Filter = "XML files (*.xml)|*.xml"
        '|All files (*.*)|*.*";
        saveDlg.AddExtension = True
        saveDlg.DefaultExt = ".xml"
        saveDlg.InitialDirectory = Application.ExecutablePath

        If saveDlg.ShowDialog() = DialogResult.OK Then
            Dim serializer As New TreeListViewSerializer()
            serializer.SaveToXml(Me.treeListView1, saveDlg.FileName, SerializationType.Style)
        End If
    End Sub

    Private Sub cbPart_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) Handles cbPart.SelectedIndexChanged
        cbState.Items.Clear()

        Select Case cbPart.SelectedIndex
            Case 1
                ' Column
                cbState.Items.Add("Hovered")
                cbState.Items.Add("Normal")
                cbState.Items.Add("Selected")
                cbState.SelectedIndex = 1
                cbState.Enabled = True
                Exit Select

                ' Item
            Case 2, 3
                ' SubItem
                cbState.Items.Add("Disabled")
                cbState.Items.Add("Focused")
                cbState.Items.Add("Hovered")
                cbState.Items.Add("Normal")
                cbState.Items.Add("Selected")
                cbState.SelectedIndex = 3
                cbState.Enabled = True
                Exit Select
            Case Else

                ' General
                cbState.Text = [String].Empty
                cbState.Enabled = False
                Exit Select
        End Select

        UpdatePropertyGrid()
    End Sub

    Private Sub rbStyle_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles rbColor.CheckedChanged, rbFormat.CheckedChanged
        UpdatePropertyGrid()
    End Sub

    Private Sub cbState_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) Handles cbState.SelectedIndexChanged
        UpdatePropertyGrid()
    End Sub

    Private Sub UpdatePropertyGrid()
        Select Case cbPart.SelectedIndex
            Case 1
                ' Column
                If rbColor.Checked Then
                    ' Color Styles
                    Select Case cbState.SelectedIndex
                        Case 0
                            ' Hovered State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.HoverColumnStyle
                            Exit Select

                        Case 2
                            ' Selected State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.SelectedColumnStyle
                            Exit Select
                        Case Else

                            ' Normal State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.NormalColumnStyle
                            Exit Select
                    End Select
                Else
                    ' Format Style
                    Me.propertyGrid1.SelectedObject = Me.treeListView1.ColumnFormatStyle
                End If
                Exit Select

            Case 2
                ' Item
                If rbColor.Checked Then
                    ' Color Styles
                    Select Case cbState.SelectedIndex
                        Case 0
                            ' Disabled State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.DisabledNodeStyle
                            Exit Select

                        Case 1
                            ' Focused State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.FocusedNodeStyle
                            Exit Select

                        Case 2
                            ' Hovered State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.HoverNodeStyle
                            Exit Select

                        Case 4
                            ' Selected State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.SelectedNodeStyle
                            Exit Select
                        Case Else

                            ' Normal State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.NormalNodeStyle
                            Exit Select
                    End Select
                Else
                    ' Format Style
                    Me.propertyGrid1.SelectedObject = Me.treeListView1.NodeFormatStyle
                End If
                Exit Select

            Case 3
                ' SubItem
                If rbColor.Checked Then
                    ' Color Styles
                    Select Case cbState.SelectedIndex
                        Case 0
                            ' Disabled State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.DisabledSubItemStyle
                            Exit Select

                        Case 1
                            ' Focused State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.FocusedSubItemStyle
                            Exit Select

                        Case 2
                            ' Hovered State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.HoverSubItemStyle
                            Exit Select

                        Case 4
                            ' Selected State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.SelectedSubItemStyle
                            Exit Select
                        Case Else

                            ' Normal State
                            Me.propertyGrid1.SelectedObject = Me.treeListView1.NormalSubItemStyle
                            Exit Select
                    End Select
                Else
                    ' Format Style
                    Me.propertyGrid1.SelectedObject = Me.treeListView1.SubItemFormatStyle
                End If
                Exit Select
            Case Else

                ' General
                If rbColor.Checked Then
                    Me.propertyGrid1.SelectedObject = Me.treeListView1.ColorStyle
                Else
                    Me.propertyGrid1.SelectedObject = Me.treeListView1.FormatStyle
                End If
                Exit Select
        End Select
    End Sub

    Private Sub btnReset_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnReset.Click
        Me.treeListView1.SuspendUpdate()

        Me.treeListView1.ColorStyle.Reset()
        Me.treeListView1.FormatStyle.Reset()
        Me.treeListView1.ColumnFormatStyle.Reset()
        Me.treeListView1.HoverColumnStyle.Reset()
        Me.treeListView1.NormalColumnStyle.Reset()
        Me.treeListView1.SelectedColumnStyle.Reset()
        Me.treeListView1.DisabledNodeStyle.Reset()
        Me.treeListView1.FocusedNodeStyle.Reset()
        Me.treeListView1.HoverNodeStyle.Reset()
        Me.treeListView1.NormalNodeStyle.Reset()
        Me.treeListView1.SelectedNodeStyle.Reset()
        Me.treeListView1.DisabledSubItemStyle.Reset()
        Me.treeListView1.FocusedSubItemStyle.Reset()
        Me.treeListView1.HoverSubItemStyle.Reset()
        Me.treeListView1.NormalSubItemStyle.Reset()
        Me.treeListView1.SelectedSubItemStyle.Reset()

        Me.treeListView1.ResumeUpdate()

        UpdatePropertyGrid()
    End Sub
#End Region

End Class
